'// SmartKill (SKILL): IPAD Mail Processing Directory Scan and Clean Tool
'// Created August 2002 by Casey Sprangel in PowerBasic/CC version 5.0 (http://www.powerbasic.com)
'//===========================================================================
'// Possible Things To Add:
'// = Move to temp directory (or back) instead of erase
'// + External file for search text allowing longer search text that may optionally include reserved command line characters
'// + Real expression (wildcard) scanning
'// + Message max size (kill/bounce/truncate messages larger than X bytes)
'// + Virus scanning (internal simplified or external)
'// + Attachment file name quoting (i.e. .EXE -> !EXE)
'// + Overload protection (i.e. if there are > X messages in a single directory, move some of them out of the way)
'// + External scripting to control functions
'// + Command line special character encoding (%##)
'// + Automatic rescanning (looping) option with delay to not hurt IPAD performance
'// + Auto-Detect email patterns across multiple messages (i.e. spam/viri) without hurting list server mail
'// + Change destination email address (SMTP envelope only) to include/exclude temp part of address
'// + Support internal FTP access to files where SMB/CIFS connections are not possible
'//===========================================================================

#COMPILE EXE                                                  '// make stand-alone program
#BREAK ON                                                     '// allow ctrl-break to abort program
#DIM ALL                                                      '// variables must be declared before use
#INCLUDE "WIN32API.INC"                                       '// include the Windows API calls for fast directory scanning

$title = "SKILL "                                             '// program name, must include the trailing space
%myver = 20081104

'// Function Declarations
DECLARE FUNCTION deltree (BYVAL PATH AS STRING) AS LONG
DECLARE FUNCTION isdir(a AS STRING) AS LONG
DECLARE FUNCTION fmove(a AS STRING) AS LONG
DECLARE FUNCTION msglock (makeit AS LONG, dname AS STRING) AS LONG
DECLARE FUNCTION rename (BYVAL filespec1 AS STRING, BYVAL filespec2 AS STRING) AS LONG
DECLARE FUNCTION scandir(BYVAL mypath AS STRING) AS LONG
DECLARE FUNCTION scanfile (FileName AS STRING, IgnoreCase AS LONG, kill8 AS LONG) AS LONG
DECLARE FUNCTION wildmatch(BYVAL filename AS STRING, BYVAL wildcard AS STRING) AS LONG
DECLARE FUNCTION zap(a AS STRING) AS LONG

'// Static Values
%Busy = 0                                                     '// flag for locking
%Unbusy = 1                                                   '// flag for unlocking
'%True = -1                                                    '// logical True in integer terms (defined in WINAPI)
'%False = 0                                                    '// logical False

'// Global variables
GLOBAL verbose AS LONG
GLOBAL totalgone AS LONG                                      '// count number of messages we remove
GLOBAL findme AS STRING                                       '// text to find inside of message
GLOBAL ignore AS LONG                                         '// flag to show if we need to ignore case
GLOBAL kill8 AS LONG                                          '// flag to show if we need to kill 8-bit text
GLOBAL moveit AS LONG                                         '// flag to show if we need to move email
GLOBAL tempdir AS STRING                                      '// holds the path to the temp directory

'//---------------------------------------------------------------------------
FUNCTION scandir(BYVAL mypath AS STRING) AS LONG              '// recurisvely scan a directory and all subdirectories
  LOCAL h AS DWORD                                            '// directory handle
  LOCAL d AS WIN32_FIND_DATA                                  '// structure for directory entry
  LOCAL f AS ASCIIZ * %max_path
  LOCAL buf AS STRING

  mypath = RTRIM$(mypath, "\") + "\"                          '// make sure path ends in a slash
  f = mypath + "*.*"                                          '// get file mask to match everything
  h = FindFirstFile(f, d)                                     '// find first file (opens a directory handle) and put result in d
  DO                                                          '// start looping
    IF BIT(d.dwFileAttributes, 4) THEN                        '// is this a directory?
      IF d.cFileName <> "." AND d.cFileName <> ".." THEN      '// skip the placeholder entries
        IF UCASE$(mypath + d.cFileName) <> UCASE$(tempdir) THEN '// filter out our own temp directory if it falls inside our search path
          scandir(mypath + d.cFileName)                       '// recursively scan this subdirectory too
        END IF
      END IF
    END IF
  LOOP WHILE FindNextFile(h, d)                               '// find next file
  FindClose h                                                 '// release the directory handle
  CONSOLE NAME $title + mypath                                '// show progress on the title bar
  STDOUT "Scanning " + mypath + "*.d"
  f = mypath + "*.d"                                          '// get message body file mask
  h = FindFirstFile(f, d)                                     '// find first file (opens a directory handle) and put result in d
  DO
    IF BIT(d.dwFileAttributes, 4)=0 THEN                      '// is this a file?
      buf = mypath + d.cFileName                              '// add path back to file name
      STDOUT "Locking " + buf
      IF msglock(%Busy, buf) THEN                             '// can we lock the message?
        '// message is locked and we have control of it.
        STDOUT "Scanning " + buf + " for " + $DQ + findme + $DQ
        IF scanfile(buf, ignore, kill8) THEN                  '// we found a match?
          STDOUT TIME$ + " Found match in " + buf
          INCR totalgone
          IF NOT moveit THEN                                  '// kill the files?
           zap LEFT$(buf, LEN(buf) -1) + "inf"                '// erase all parts of this message
           zap LEFT$(buf, LEN(buf) -1) + "x"                  '// erase all parts of this message
           zap LEFT$(buf, LEN(buf) -1) + "d"                  '// erase all parts of this message
           zap LEFT$(buf, LEN(buf) -1) + "cmd"                '// erase all parts of this message
           zap LEFT$(buf, LEN(buf) -1) + "cmz"                '// erase all parts of this message
           zap LEFT$(buf, LEN(buf) -1) + "bsy"                '// must be last to go
          ELSE                                                '// move the files, don't kill them
           fmove LEFT$(buf, LEN(buf) -1) + "inf"
           fmove LEFT$(buf, LEN(buf) -1) + "x"
           fmove LEFT$(buf, LEN(buf) -1) + "d"
           fmove LEFT$(buf, LEN(buf) -1) + "cmd"
           fmove LEFT$(buf, LEN(buf) -1) + "cmz"
           fmove LEFT$(buf, LEN(buf) -1) + "bsy"              '// must be last to go
          END IF                                              '// kill the files?
        ELSE                                                  '// nothing found in this message
          STDOUT "Unlocking " + buf
          IF msglock(%Unbusy, buf) = %False THEN              '// can't unlock the message?
           STDOUT "Error: Unable to unlock message " + buf
          END IF                                              '// could we unlock message?
        END IF                                                '// found text match?
      END IF                                                  '// could we lock message?
    END IF                                                    '// is this a file
  LOOP WHILE FindNextFile(h, d)                               '// find next file
  FindClose h                                                 '// release the directory handle
END FUNCTION

'//---------------------------------------------------------------------------
FUNCTION scanfile (MsgName AS STRING, IgnoreCase AS LONG, kill8 AS LONG) AS LONG
  DIM handle1 AS LOCAL LONG
  DIM buf AS LOCAL STRING
  DIM buf1 AS LOCAL STRING
  DIM found AS LOCAL LONG
  DIM i AS LOCAL LONG

  found = %False                                              '// start out with nothing found
  handle1 = FREEFILE                                          '// get next available file handle
  IF ISFILE(MsgName) THEN                                     '// does the file exist?
    OPEN MsgName FOR BINARY AS handle1                        '// open file in binary mode
    i = LOF(handle1)                                          '// reset chunk to load whole file (may need work on very large messages, but fast enough for now)
    buf1 = SPACE$(i)                                          '// create buffer for input
    GET #handle1, , buf1                                      '// get chunk of file
    CLOSE handle1                                             '// release file handle
    IF INSTR(buf1, findme) > 0 THEN                           '// is string found?
      found = %True                                           '// we found what we were looking for
    END IF
    IF kill8 THEN                                             '// are we looking for 8-bit characters?
      IF INSTR(buf1, ANY CHR$(128 TO 255)) > 0 THEN           '// are any of the high ASCII characters in our message chunk?
        found = %True                                         '// we found what we are looking for
      END IF
    END IF
    IF IgnoreCase THEN                                        '// ignore search case?
      IF INSTR(LCASE$(buf1), LCASE$(findme)) > 0 THEN
        found = %True                                         '// we found what we were looking for
      END IF
    END IF                                                    '// ignore search case?
    buf1 = ""                                                 '// release string memory
  END IF
  FUNCTION = found                                            '// return what we found
END FUNCTION

'//-------------------------------------------------------------------------
FUNCTION rename (BYVAL filespec1 AS STRING, BYVAL filespec2 AS STRING) AS LONG
  DIM fs1 AS LOCAL STRING                                     '// local strings because we will modify
  DIM fs2 AS LOCAL STRING

  IF RIGHT$(filespec1, 2) = ":\" THEN                         '// source is root?
    IF verbose THEN
      STDOUT "* Error: Source is root. Can not move."
    END IF
    EXIT FUNCTION
  END IF
  IF RIGHT$(filespec2, 2) = ":\" THEN                         '// destination is root?
    IF verbose THEN
      STDOUT "* Error: Destination is root. Can not move."
    END IF
    EXIT FUNCTION
  END IF
  IF MID$(filespec1, 2, 1) = ":" THEN                         '// a drive letter is specified
    IF MID$(filespec2, 2, 1) = ":" THEN                       '// both contain a drive letter
      IF LEFT$(filespec1, 1) <> LEFT$(filespec2, 1) THEN      '// drive letters don't match
        IF verbose THEN
          STDOUT "* Error: failed rename across volumes " + filespec1 + " " + filespec2
        END IF
        FUNCTION = %False                                     '// say we failed, we know we will
        EXIT FUNCTION                                         '// we are done, leave
      END IF                                                  '// source and destination drives don't match
    END IF                                                    '// both contain a drive letter
  END IF                                                      '// source has a drive letter
  IF ISFILE(filespec2) THEN                                   '// does destination file exist?
    IF NOT zap(filespec2) THEN                                '// not able to erase destination file?
      STDOUT "* Unable to overwrite/erase " + filespec2
      FUNCTION = %False
      EXIT FUNCTION                                           '// leave, we can't rename/move
    END IF
  END IF
  ON ERROR RESUME NEXT                                        '// turn off error traps
  NAME filespec1 AS filespec2                                 '// rename the file (may move between drives)
  ON ERROR GOTO 0                                             '// turn on error traps
  IF ISFILE(filespec2) THEN                                   '// yes the file is renamed
   FUNCTION = %True                                           '// we think it worked
  ELSE
   FUNCTION = %False                                          '// we think we failed
  END IF
END FUNCTION

'//---------------------------------------------------------------------------
FUNCTION zap (a AS STRING) AS LONG                            '// Attempts to delete a file and return success or failure
  IF ISFILE(a) THEN                                           '// does file exist?
    IF (GETATTR(a) AND 16) THEN                               '// it is a directory (not a file)?
      IF verbose THEN                                         '// tell user extra output?
        STDOUT "  " + a + " looks like a directory, using DELTREE."
        FUNCTION = deltree(a)                                 '// pass to deltree function
      END IF
    ELSE                                                      '// this is a file
      IF UCASE$(a) = "IPADLANG.IDX" THEN                      '// not allowed to erase this file
        FUNCTION = %False                                     '// return that we failed
        EXIT FUNCTION                                         '// leave this function
      END IF
      IF verbose THEN
        STDOUT "  Erasing file " + a
      END IF
      ON ERROR RESUME NEXT                                    '// turn off error checking for a while
      KILL a                                                  '// delete the file
      ON ERROR GOTO 0                                         '// resume normal error checking
      IF ISFILE(a) THEN                                       '// does file exist?
        STDOUT "  Error, file not erased " + a
        FUNCTION = %False                                     '// we could not erase the file
      ELSE                                                    '// it worked, file is not here
        FUNCTION = %True                                      '// return true - file is now gone
      END IF
    END IF
  ELSE                                                        '// hey! file is not here, we win!
    FUNCTION = %True                                          '// return true
  END IF
END FUNCTION

'//---------------------------------------------------------------------------
FUNCTION msglock (makeit AS LONG, dname AS STRING) AS LONG    '// Attempts to lock/unlock a message file set and return success or failure
  LOCAL dname1 AS STRING                                      '// root message file name
  LOCAL dname2 AS STRING
  LOCAL i AS LONG
  IF INSTR(dname, ".") > 0 THEN                               '// does file name contain a period?
    dname1 = LEFT$(dname, INSTR(-1, dname, "."))              '// cut off after last period
  ELSE                                                        '// file name does not contain a period
    dname1 = dname + "."                                      '// just copy file name with period at end
  END IF                                                      '// does file name contain a period?
  SELECT CASE makeit                                          '// what do we do with the file?
   CASE %Busy                                                 '// we should lock message
     IF ISFILE(dname1 + "bsy") THEN                           '// is message already locked?
       IF ISFILE(dname1 + "cmd") THEN                         '// double state message
         zap dname1 + "cmd"                                   '// erase the cmd version
       END IF
       IF ISFILE(dname1 + "cmz") THEN                         '// double state sleepy message
         zap dname1 + "cmz"                                   '// erase the sleepy cmd version
       END IF
       STDOUT "  Message " + dname1 + " is already locked. Skipping."
       FUNCTION = %False                                      '// then we failed
       EXIT FUNCTION                                          '// leave here
     END IF
     IF NOT ISFILE(dname1 + "cmd") AND NOT ISFILE(dname1 + "cmz") THEN '// message set is not complete
       i = FREEFILE                                           '// get next file available handle
       OPEN dname1 + "bsy" FOR OUTPUT AS i                    '// create the CMD file
       ?#i, "S 0" + RTRIM$(dname1, ".") + ".D D.ipads" + RTRIM$(dname1, ".") + " xx.x - D.ipads" + RTRIM$(dname1, ".") + " 666"
       ?#i, "S 0" + RTRIM$(dname1, ".") + ".X X.ipads" + RTRIM$(dname1, ".") + " xx.x - X.ipads" + RTRIM$(dname1, ".") + " 666"
       CLOSE i                                                '// release file handle
       IF verbose THEN
        STDOUT "[I] Created missing " + dname1 + "CMD"
       END IF
       FUNCTION = %True                                       '// we made the file, so we worked
       EXIT FUNCTION                                          '// leave function
     END IF
     dname2 = dname1 + "bsy"                                  '// destination file name
     dname1 = dname1 + "cmd"                                  '// source file name
   CASE %Unbusy                                               '// we should unlock message
     IF ISFILE(dname1 + "cmd") THEN                           '// double state message?
       zap dname1 + "cmd"                                     '// erase the other CMD file
     END IF
     IF ISFILE(dname1 + "cmz") THEN                           '// double state sleepy message?
       zap dname1 + "cmz"                                     '// erase the other CMD file
     END IF
     dname2 = dname1 + "cmd"                                  '// destination file name
     dname1 = dname1 + "bsy"                                  '// source file name
   CASE ELSE                                                  '// programmer made an error?
     STDOUT ">> Programmer error: unknown case option in msglock function!"
     FUNCTION = %False                                        '// either way we failed
     EXIT FUNCTION
   END SELECT
   IF ISFILE(dname2) THEN                                     '// destination name exists
     STDOUT "  Rename destination file name already exists."
     STDOUT "  (" + dname2 + ")"
     FUNCTION = %False                                        '// set to failure mode
     EXIT FUNCTION                                            '// leave here
   END IF
   rename dname1, dname2                                      '// try to rename file
   IF ISFILE(dname1) THEN                                     '// does original file still exist?
     STDOUT "  Rename failed for " + dname1
     FUNCTION = %False                                        '// we could not rename the file
   ELSE                                                       '// it worked, source file is not here
     FUNCTION = %True                                         '// return true - source file is now gone
     IF NOT ISFILE(dname2) THEN                               '// new file name not found?
       STDOUT "  Error: source and destination missing after rename. Data lost?"
       STDOUT "  (" + dname1 + " / " + dname2 + ")"
       FUNCTION = %False                                      '// strange, BOTH source and destination are gone!
     END IF
  END IF
END FUNCTION

'//---------------------------------------------------------------------------
FUNCTION wildmatch(BYVAL matchname AS STRING, BYVAL wildcard AS STRING) AS LONG
  '// Wildcard string compare. Returns %True=-1 or %False=0
  DIM FilePos AS LOCAL LONG
  DIM WildPos AS LOCAL LONG
  DIM FileByte AS LOCAL LONG
  DIM WildByte AS LOCAL LONG

  IF WildCard$ = "*.*" OR WildCard$ = "*" THEN                '// "*.*" matches everything
    FUNCTION = %True                                          '// it matches
    EXIT FUNCTION                                             '// leave
  END IF
  matchName$ = UCASE$(matchName$)                             '// Convert strings to upper case
  WildCard$ = UCASE$(WildCard$)
  FilePos = 1                                                 '// start pointers at first character
  WildPos = 1
  DO                                                          '// start unconditional loop
    FileByte = ASC(MID$(matchName$, FilePos))                 '// Get one byte from each string
    WildByte = ASC(MID$(WildCard$, WildPos))
    IF WildPos > LEN(WildCard$) THEN                          '// End of wildcard?  See if we have a match
      FUNCTION = (FilePos >= LEN(matchName$))                 '// are we at the end of the string?
      EXIT FUNCTION                                           '// leave
    ELSEIF FilePos > LEN(matchName$) THEN                     '// End of filename?  No match
      FUNCTION = %False                                       '// not a match
      EXIT FUNCTION                                           '// leave
    ELSEIF (WildByte = FileByte) OR (WildByte = 63) THEN      '// Do bytes match?  Or is WildByte a question mark?
      INCR WildPos                                            '// add one to the wildcard pointer
      INCR FilePos                                            '// add one to the file name pointer
    ELSEIF (WildByte = 42) THEN                               '// Is WildByte an asterisk? (matches everything)
      WHILE (FileByte <> 32) AND (FileByte <> 46) AND (FilePos < LEN(matchName$)) '// Skip to period or end of filename
        INCR FilePos                                          '// add one to the file name pointer
        FileByte = ASC(MID$(matchName$, FilePos))
      WEND
      INCR WildPos                                            '// move wildcard poiner
    ELSE                                                      '// No match, so exit
      FUNCTION = %False
      EXIT FUNCTION                                           '// leave
    END IF                                                    '// end of wildcard?
  LOOP
END FUNCTION

'//---------------------------------------------------------------------------
FUNCTION deltree (BYVAL mypath AS STRING) AS LONG             '// Recursively removes a subdirectory and all child directories
  LOCAL h AS DWORD                                            '// directory handle
  LOCAL d AS WIN32_FIND_DATA                                  '// structure for directory entry
  LOCAL f AS ASCIIZ * %max_path

  mypath = RTRIM$(mypath, "\")                                '// remove any trailing slash marks from path
  IF LEN(mypath) = 2 AND RIGHT$(mypath, 1) = ":" THEN         '// root directory?
    IF verbose THEN
      STDOUT "* Error: Unable to deltree root " + mypath
    END IF
    EXIT FUNCTION                                             '// leave, we can't do anything to the root
  END IF
  IF ISFILE(mypath) THEN                                      '// does this file/path even exist?
    IF verbose THEN
      STDOUT "  Removing directory " + mypath
    END IF
    IF GETATTR(mypath) AND 16 THEN                            '// is this a directory?
      mypath = RTRIM$(mypath, "\") + "\"                      '// make sure path ends in a slash
      f = mypath + "*.*"                                      '// get file mask
      h = FindFirstFile(f, d)                                 '// find first file (opens a directory handle) and put result in d
      DO
        IF BIT(d.dwFileAttributes, 4) THEN                    '// is this a directory?
          IF d.cFileName <> "." AND d.cFileName <> ".." THEN  '// skip the placeholder entries
            FUNCTION = deltree(mypath + d.cFileName)          '// delete that directory too
          END IF
        ELSE                                                  '// this is a file
          IF NOT zap(mypath + d.cFileName) THEN               '// not able to erase file?
            FUNCTION = %False                                 '// can't continue if we can't erase file
            EXIT FUNCTION                                     '// leave
          END IF
        END IF                                                '// is this a directory
      LOOP WHILE FindNextFile(h, d)                           '// find next file
      FindClose h                                             '// release the directory handle
      ON ERROR RESUME NEXT                                    '// turn off error checking
      RMDIR mypath                                            '// try to remove this path
      ON ERROR GOTO 0                                         '// turn on error checking
      IF ISFILE(mypath) THEN                                  '// is the directory still here?
        FUNCTION = %False                                     '// we failed
      ELSE                                                    '// Yes! Directory is not here!
        FUNCTION = %True                                      '// we made it this far, things are good
      END IF
    ELSE                                                      '// this is a file, not a directory
      FUNCTION = zap(mypath)                                  '// remove file and return success/fail
    END IF
  END IF                                                      '// does path exist?
END FUNCTION

'//---------------------------------------------------------------------------
FUNCTION isdir (a AS STRING) AS LONG
  IF ISFILE(RTRIM$(a,"\")) THEN                               '// does it exist? (file or dir)
    IF verbose THEN                                           '// are we in verbose mode
      STDOUT "Attribute for " + a + " is " + FORMAT$(GETATTR(RTRIM$(a,"\")))
    END IF
    FUNCTION = (GETATTR(RTRIM$(a,"\")) AND 16) = 16          '// return true if this is a directory
  ELSE                                                        '// does not exist
    FUNCTION = %False                                         '// return false
  END IF
END FUNCTION

'//---------------------------------------------------------------------------
FUNCTION fmove (a AS STRING) AS LONG                          '// moves a file to a temp subdirectory using the RENAME function above
  LOCAL buf AS STRING
  LOCAL i AS LONG
  i = INSTR(-1, buf, "\")                                     '// find offset to last slash mark
  IF i > 0 THEN                                               '// is there a path in this string?
    buf = RTRIM$(tempdir, "\")+ "\" + MID$(a, i + 1)          '// add new path
  ELSE                                                        '// no path on string?
    buf = RTRIM$(tempdir, "\")+ "\" + a
  END IF
  rename a, buf                                               '// call the rename function to do the work
  FUNCTION = ISFILE(buf)                                      '// return true if the file exists in new location
END FUNCTION

'//---------------------------------------------------------------------------
FUNCTION PBMAIN()                                             '// life starts here.
  DIM buf AS STRING                                           '// temp buffer
  DIM c AS STRING                                             '// holds the command line while we work on it
  DIM handle1 AS LONG                                         '// file handle
  DIM i AS LONG                                               '// temp counter
  DIM mypath AS STRING                                        '// working path

  buf = COMMAND$                                              '// get a copy of the command line
  STDOUT $title + FORMAT$(%myver)
  IF LEN(buf) = 0 OR LEFT$(buf, 2) = "/?" THEN                '// user asked for help?
    STDOUT "Smart Email Kill"
    STDOUT ""
    STDOUT "This utility will scann mail processing files on an IPAD which contain"
    STDOUT "a line of text which you specify. This scan is only done on the .D file"
    STDOUT "which is the header and body of the message and not the envelope. If"
    STDOUT "there is a text match then all mail files with the same root name"
    STDOUT "(the same first 8 characters) will be erased."
    STDOUT ""
    STDOUT "This utility will only scan messages which are not locked Busy by the"
    STDOUT "IPAD or other mail processing utility. This should allow the scanning"
    STDOUT "to be done through a LAN connection to the IPAD while it is running."
    STDOUT ""
    STDOUT "Command line syntax:"
    STDOUT "SKILL <path> [/8] [/I] [/V] [/M[=<path>]] /S=<text>"
    STDOUT "   <path> is the fully qualified path to the mail directory. This may be"
    STDOUT "          a valid UNC style path or a Windows style path."
    STDOUT "     /8 = Detect and remove messages with illegal 8-bit data in the message"
    STDOUT "          body text."
    STDOUT "     /I = Ignore case. This makes the text search case insensitive where"
    STDOUT "          UPPERCASE and lowercase letters are considered the same."
    STDOUT "     /V = Verbose mode. Display extra output to the screen."
    STDOUT "/M[=<path>] = Move matching files to a temp dir instead of erasing them."
    STDOUT "          Default is (TEMP) if the optional file name is not given."
    STDOUT "     /S = The search option is immediatly followed by an equal sign (=)"
    STDOUT "          and the search text which must be the last option on the line."
    EXIT FUNCTION                                             '// leave program
  END IF
  mypath = PARSE$(buf, ANY " /", 1)                           '// get path from command line
  buf = LTRIM$(MID$(buf,1 + LEN(mypath)))                     '// remove path from command line
  IF INSTR(mypath, "\") = 0 THEN                              '// no path information?
    mypath = TRIM$(CURDIR$,"\") + "\" + mypath                '// add current directory
  END IF
  mypath = RTRIM$(mypath, "\") + "\"                          '// ensure only one trailing slash
  DO WHILE LEFT$(buf,1) = "/"                                 '// loop through all command line switches
    c = LEFT$(buf,2)                                          '// grab the left-most two characters
    SELECT CASE UCASE$(c)                                     '// uppercase the switch options
    CASE "/V"
      verbose = %True                                         '// display extra output
      buf = LTRIM$(MID$(buf, 3))                              '// cut off the switch
    CASE "/I"
      ignore = %True                                          '// ignore search text case
      buf = LTRIM$(MID$(buf, 3))                              '// cut off the switch
    CASE "/M"
      moveit = %True                                          '// move files, no killing
      IF MID$(buf,3,1) = "=" THEN                             '// did user provide the equal sign?
        tempdir = MID$(buf, 4)                                '// get the directory
        IF LEFT$(tempdir, 1) = $DQ THEN                       '// this is a quoted filename
          tempdir = LEFT$(tempdir, INSTR(2, tempdir + $DQ, $DQ)) '// find end quote or end of string
        ELSE
          tempdir = LEFT$(tempdir, INSTR(tempdir + $SPC, ANY "/" + $SPC) - 1) '// find space, next switch or end of string
        END IF
        buf = LTRIM$(MID$(buf, 4 + LEN(tempdir)))             '// cut off switch with path
        tempdir = RTRIM$(TRIM$(tempdir, ANY $DQ + $SPC), "\") '// remove any leading and trailing spaces and double quotes and any trailing slashes
      ELSE                                                    '// user did not give path
        tempdir = mypath + "TEMP"                             '// default temp directory
        buf = LTRIM$(MID$(buf, 3))                            '// cut off the switch
      END IF
    CASE "/S"                                                 '// search switch
      findme = MID$(buf, 4)                                   '// copy command line into find string
      buf = ""                                                '// forget rest of command line, we are done
    CASE "/8"                                                 '// look for 8-bit data?
      kill8 = %True
      buf = LTRIM$(MID$(buf, 3))                              '// cut off the switch
    CASE ELSE
      STDOUT "* Error: Unknown command line switch argument " + $DQ + c + $DQ
      FUNCTION = 1                                            '// set errorlevel 1
      EXIT FUNCTION                                           '// leave the program
    END SELECT
    c = PARSE$(buf, ANY " /", 1)                              '// get next option from command line
  LOOP
  IF NOT isdir(mypath) THEN                                   '// is the path not here?
    IF ISFILE(RTRIM$(mypath, "\")) THEN                       '// this looks like a file name
      STDOUT "* Error: " + $DQ + mypath + $DQ + " is a file, not a directory."
    ELSE
      STDOUT "* Error: Path not found " + $DQ + mypath + $DQ
    END IF
   FUNCTION = 1                                               '// set errorlevel 1
   EXIT FUNCTION                                              '// leave program
  END IF                                                      '// is the path not here
  IF verbose THEN
    STDOUT "Command line: " +  COMMAND$
    STDOUT "Find string: " + $DQ + findme + $DQ
    IF ignore THEN
      STDOUT "Case Insensitive search"
    ELSE
      STDOUT "Case Sensitive"
    END IF
    STDOUT "In path: " + mypath
    IF moveit THEN
      IF NOT ISFILE(tempdir) THEN                             '// temp directory does not exist?
        MKDIR tempdir                                         '// make temp directory below the path
      END IF
      STDOUT "Matching messages will be moved to " + tempdir
    END IF
    IF kill8 THEN
      STDOUT "All messages with 8-bit data (non-RFC822 compliant) will be erased."
    END IF
  END IF
  scandir(mypath)                                             '// do the work
  IF totalgone = 0 THEN                                       '// was anything erased
    STDOUT "No messages were " + IIF$(moveit, "moved to " + tempdir, "erased") + "."
  ELSE
    STDOUT FORMAT$(totalgone) + " message" + IIF$(totalgone = 1, "", "s") + " matched and " + IIF$(moveit, "moved to " + tempdir, "erased") + "."
  END IF
END FUNCTION                                                  '// leave program
